from dataclasses import dataclass
from datetime import datetime
from typing import List, Protocol


@dataclass
class Transaction:
    date: datetime
    amount: int
    payee_name: str
    description: str


class IWriteRepository(Protocol):
    def create_many(self, transactions: List[Transaction]) -> int:
        ...


class IReadRepository(Protocol):
    def read_transactions(self) -> List[Transaction]:
        ...


def import_transactions(read: IReadRepository, write: IWriteRepository) -> int:
    return write.create_many(read.read_transactions())



from dataclasses import dataclass
from typing import List

from dataclass_csv import DataclassReader

from ynab_import.common.providers import CSVReader
from ynab_import.core.interactions import Transaction
from ynab_import.swedbank.models import SwedbankTransaction


@dataclass
class SwedbankCSVReader(CSVReader):
    CLIENT_ACCOUNT = "Client account"
    DATE = "Date"
    PAYEE = "Beneficiary/Payer"
    MEMO = "Details"
    AMOUNT = "Amount"
    DEBIT_CREDIT = "Debit/Credit"

    SKIP_KEYWORDS = ["closing balance", "Turnover", "Opening balance"]

    source_file_path: str

    def read_transactions(self) -> List[Transaction]:
        with open(self.source_file_path, "r", encoding="utf8") as f:
            reader = DataclassReader(
                f, SwedbankTransaction, delimiter=";", quotechar='"'
            )
            reader.map(self.CLIENT_ACCOUNT).to("client_account")
            reader.map(self.DATE).to("date")
            reader.map(self.PAYEE).to("payee")
            reader.map(self.MEMO).to("memo")
            reader.map(self.AMOUNT).to("amount")
            reader.map(self.DEBIT_CREDIT).to("debit_credit")
            swedbank_transactions = list(reader)
        return [
            self._make(tr)
            for tr in swedbank_transactions
            if tr.memo not in self.SKIP_KEYWORDS
        ]

    def _make(self, transaction: SwedbankTransaction) -> Transaction:
        amount = self.to_amount(transaction.amount)
        if transaction.debit_credit == "D":
            amount = amount * -1

        return Transaction(
            date=transaction.date,
            amount=amount,
            payee_name=transaction.payee,
            description=transaction.memo,
        )



from dataclasses import dataclass
from typing import Optional


@dataclass
class YnabTransaction:
    account_id: str
    date: str
    amount: str
    payee_name: str
    memo: str
    import_id: str

    cleared: Optional[str] = None
    approved: bool = False
    flag_color: str = "green"



from dataclasses import dataclass
from typing import Any, Dict, Protocol, TypeVar

import requests
from requests import Response

T = TypeVar("T")


class HttpClient(Protocol[T]):
    def post(self, endpoint: str, *, json: Dict[str, Any]) -> T:
        ...


@dataclass
class RequestsClient:
    def __init__(self):
        self.base_url: str = ""
        self.kwargs: Dict[str, Any] = {}

    def with_base_url(self, base_url: str) -> "RequestsClient":
        self.base_url = base_url

        return self

    def with_timeout(self, timeout: float) -> "RequestsClient":
        self.kwargs.setdefault("timeout", timeout)

        return self

    def with_header(self, key: str, value: str) -> "RequestsClient":
        headers = self.kwargs.setdefault("headers", {})
        headers[key] = value

        return self

    def with_api_key_auth(self, api_key: str) -> "RequestsClient":
        self.with_header("Authorization", f"Bearer {api_key}")

        return self

    def request(self, method: str, endpoint: str, **kwargs: Any) -> Response:
        url = f'{self.base_url.rstrip("/")}/{endpoint.lstrip("/")}'
        return requests.request(method, url, **kwargs, **self.kwargs)

    def post(self, endpoint: str, *, json: Dict[str, Any]) -> Response:
        return self.request("POST", endpoint, json=json)



from dataclasses import asdict, dataclass
from pprint import pprint
from typing import List

from requests import Response

from ynab_import.common.models import YnabTransaction
from ynab_import.core.interactions import Transaction
from ynab_import.infra.http import HttpClient


@dataclass
class YnabAPIBasedRepository:
    http_client: HttpClient[Response]
    budget_id: str
    account_id: str

    def create_many(self, transactions: List[Transaction]) -> int:
        ynab_transactions = [asdict(self._to_ynab(t)) for t in transactions]
        pprint(ynab_transactions)
        response = self.http_client.post(
            f"/budgets/{self.budget_id}/transactions",
            json={"transactions": ynab_transactions},
        )

        print(response.json())
        response.raise_for_status()
        return response.status_code

    def _to_ynab(self, transaction: Transaction) -> YnabTransaction:
        import_id = "YNAB:{milliunit_amount}:{iso_date}".format(
            milliunit_amount=transaction.amount, iso_date=transaction.date.isoformat(),
        )

        return YnabTransaction(
            account_id=self.account_id,
            date=transaction.date.isoformat(),
            amount=str(transaction.amount),
            payee_name=transaction.payee_name,
            memo=transaction.description,
            cleared="cleared",
            approved=False,
            flag_color="red",
            import_id=import_id,
        )


@dataclass
class InMemoryBasedRepository:
    transactions: List[Transaction]

    def create_many(self, transactions: List[Transaction]) -> int:
        for transaction in transactions:
            self.transactions.append(transaction)
        return len(self.transactions)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ynab_import',
 'ynab_import.common',
 'ynab_import.core',
 'ynab_import.infra',
 'ynab_import.revolut',
 'ynab_import.swedbank']

package_data = \
{'': ['*']}

install_requires = \
['dataclass-csv>=1.1,<2.0',
 'pydantic>=1.8.2,<2.0.0',
 'python-dotenv>=0.19.2,<0.20.0',
 'typer>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['ynab-import = ynab_import.__main__:app']}

setup_kwargs = {
    'name': 'ynab-import',
    'version': '0.0.5',
    'description': 'Automate transaction import troubles to ynab.',
    'long_description': '# YNAB - You Need a Budget\n\n[![Tests](https://github.com/demonno/ynab-import/workflows/Test%20Suite/badge.svg)](https://github.com/demonno/ynab-import/actions)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![version](https://img.shields.io/pypi/v/ynab-import.svg)](https://pypi.org/project/ynab-import/)\n[![license](https://img.shields.io/pypi/l/ynab-import)](https://github.com/demonno/ynab-import/blob/master/LICENSE)\n\nBudgeting Tool helps you plan and manage your personal budget.\nIn Ynab you have budgets, each budget has one or more accounts with transactions.\n\n## Run tests\n\n    python -m pytest\n\n## How to run\n\n* Clone project\n* Create .env file in project root\n\n```ini\nSOURCE=revolut\nREAD_FROM=../statement.csv\nWRITE_TO=api\n\nYNAB_API_KEY=\nYNAB_BUDGET_ID=\n\n# ISIC\n#YNAB_ACCOUNT_ID=739410aa-0ef1-4b3e-a488-4ffe5ca3209a\n\n# Checking\nYNAB_ACCOUNT_ID=\n# Saving\n#YNAB_ACCOUNT_ID=\n```  \n\n* Run `main.py` \n\n### Developer API\n\n* https://api.youneedabudget.com/    \n    \n    \n#User API\n \n     ynab-import --s swedbank -r api -w api\n     ynab-import --s swedbank -r file.csv -w file.csv\n     ynab-import --s swedbank -r file.csv -w api\n \n Some default/extra configurations required\n \n * read-from file\n    * INPUT_FILE_PATH [default (~/{bank}.csv) home@Linux] (actual full file path required)\n * read-from api\n    * appropriate bank credentials and setup required\n    * Raise Not Supported\n * write to file \n    * OUTPUT_FILE_PATH [default (~/{bank}-ynab-{date-time}.csv) home@Linux] (add file name only if not specified)\n * write to api \n    * YNAB_API_KEY\n    * YNAB_BUDGET_ID\n\nYNAB_API_BASE_URL="https://api.youneedabudget.com/v1/"\n\n\n\n#### User Wiki\n\n\n##### Ynab \n\n####### Import Manually\n\nIts posisble to import Csv File manually\n\n\n####### Import Using Ynab API\nTo set up this integration you need to create `Personal Access Token`.\n\nGo to `https://app.youneedabudget.com/settings/developer` and click `New Token` \n',
    'author': 'demonno',
    'author_email': 'demur.nodia@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/demonno/ynab-import',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
